import 'dart:typed_data';

import 'enums.dart';
import 'package:palette/palette.dart';
export 'chonkyLoc.dart';
import 'dart:convert';
import 'xxhash/lib/fast_hash.dart';
export 'moderatorCapsuleEntry.dart';
export 'moderatorCapsule.dart';

final ColorPalette _cp = ColorPalette.polyad(
  CmykColor(100, 50, 0, 0),
  numberOfColors: 127,
  hueVariability: 15,
  saturationVariability: 10,
  brightnessVariability: 3,
);
//final dynamic smileyKeys = smileys.keys.toList();
//ThicketEntry? _prpreEntry;

class ModeratorEntry {
  ThicketContainerSubType type;
  String author;
  Uint8List _body;
  Uint8List _shortlink;
  int keyHint;
  int bl;
  int pp;
  String colour;
  String avatar;
  String _nick;
  String attachmentLink;
  bool isReply = false;
  bool isPost = false;
  bool isRoot = false;
  bool pawMatch = false;
  bool blsGroupMatch = false;
  bool muted;
  bool isGroupParent = false;
  bool isHidden = false;
  bool isHiddenDecrypted = true;
  bool isAnonymous = true;
  ModeratorEntry(
      this.type,
      this.author,
      this._body,
      this._shortlink,
      this.keyHint,
      this.bl,
      this.pp,
      this.colour,
      this.avatar,
      this._nick,
      this.isReply,
      this.isRoot,
      this.isPost,
      this.isAnonymous,
      this.pawMatch,
      this.muted,
      this.isGroupParent,
      this.isHidden,
      this.isHiddenDecrypted,
      this.attachmentLink);
  Uint8List get bodyBytes => _body;
  Uint8List get shortLinkBytes => _shortlink;
  int get hint {
    //short link to crypted
    final digest = xxHash64.convert(_shortlink);
    return digest.x.toInt();
  }

  String get nick {
    if (_nick == null) {
      _nick = author.substring(0, 6);
    }
    return _nick;
  }

  int get typesetForCapsule {
    int i = 0;
    if (isPost) i += 1; //0 reply
    if (isGroupParent) i += 4;
    return i;
  }

  int get calculatedServerTimestamp => 0;
  bool get hasAttachment => attachmentLink.isEmpty ? false : true;
  String get shortLink {
    if (_shortlink.isEmpty) return 'BROKEN_SLINK';
    return base64UrlEncode(_shortlink);
    //return HtmlEscape().convert(utf8.decode(_shortlink));
  }

  String get body {
    if (_body.isEmpty) return 'BROKEN_BODY';
    return utf8.decode(_body);
  }

  void set nick(String s) => _nick = s;
  String get twitterPostTitle {
    //parse from body
    return '';
  }

  String get twitterPostContent {
    //parse from body
    //[](cid) #vv daddo
    var bo = body.split(' ');
    String s = '';
    for (var i = 2; i < bo.length; i++) {
      s += bo[i] + ' ';
    }
    return s;
  }

  String get twitterAttachmentLink {
    var bo = body.split('/');
    var bb = bo.last.split(')');
    //String cid = bo.last.substring(0, bo.last.length - 1);

    return 'https://cloudflare-ipfs.com/ipfs/' + bb.first;
  }
}

class ModeratorEntrySet {
  Map<int, ModeratorEntry> _data = {};
  Set<int> _groups = {};
  Set<int> _threads = {};
  Set<int> _posts = {};
  Set<int> _replies = {};
  Map<int, List<int>> _children = {};
  Map<int, List<int>> _groupChildren = {};
  Map<int, List<int>> _parentReplies = {};
  int idx = 0;
  ModeratorEntrySet();

  factory ModeratorEntrySet.fromList(List<ModeratorEntry> _l) {
    ModeratorEntrySet _ll = ModeratorEntrySet();
    _l.forEach((element) {
      _ll.add(element);
    });
    return _ll;
  }
  ModeratorEntry get root {
    //case where we have only one root+replies
    return _data[_posts.first]!;
  }

  List<ModeratorEntry> get all => _data.values.toList();
  void add(ModeratorEntry m) {
    final int hint = m.keyHint;

    if (_children[m.bl] == null) {
      _children[m.bl] = [];
    } else {
      _children[m.bl]!.add(m.keyHint);
    }
    if (m.isGroupParent) {
      _groups.add(m.keyHint);
      _children[m.keyHint] = [];

      _groupChildren[m.keyHint] = [];
    } else {
      if (m.isPost) {
        _threads.add(m.keyHint);
      }
    }
    if (m.isPost) {
      _posts.add(m.keyHint);
    }
    if (m.isReply) {
      _replies.add(m.keyHint);
      if (_parentReplies[m.bl] == null) {
        _parentReplies[m.bl] = [];
      }
      _parentReplies[m.bl]!.add(m.keyHint);
    }

    _data[m.keyHint] = m;
  }

  List<ModeratorEntry> get groups {
    List<ModeratorEntry> _g = [];
    _groups.forEach((element) {
      _g.add(_data[element]!);
    });
    return _g;
  }

  List<ModeratorEntry> get threads {
    List<ModeratorEntry> _g = [];
    _threads.forEach((element) {
      _g.add(_data[element]!);
    });
    return _g;
  }

  List<ModeratorEntry> childrenOf(int parent) {
    List<ModeratorEntry> _g = [];
    if (_children[parent] == null) return _g;

    _children[parent]!.forEach((element) {
      _g.add(_data[element]!);
    });
    return _g;
  }
}

/*
class ThicketEntryHelper {
  Tentry? parse(ThicketEntry t, Set<int> _group) {
    if (!t.isSns) return null;
    
    tp.author = t.author.base58;
    tp.body = t.payloadAsString;
    // markdownToHtml(t.payloadAsString);
    tp.isReply = t.isReply;
    tp.link = t.entryHash.base58;
    if (tp.backLink != null) {
      if (_group.contains(t.backLink.hint)) {
        tp.backLink = t.backLink.base58;
      }
    }
    return Tentry(type, author, body, link, backLink, colour, avatar, _nick)
  }
}*/
