import 'dart:convert';
import 'dart:ffi';
import 'dart:typed_data';

class ModeratorCapsuleItem {
  //everything that is 0 means we dont know. author,parent,pp,timestamp missing. this is normal.
  //max entry size 2kb
  //if if does not fit 1kb payload, its off tangle and needs a CID
  Uint8List flags; //thicket internal type. factory sealed, do not open.
  int authorHint;
  int bl; //parent entry key if we know it
  int pp; //what looks like the previous entry
  int garyTimestamp; //obvious.
  int payloadLenght; //how many bytes for payload
  Uint8List payload; //the groovy stuff
  int keyLen;
  Uint8List key; //item key when talking to moderator

  int hint;
  int lenghtInBytes;
  String authorAvatar;
  String authorNick; //who authored this 3 mins ago then switched id
  Uint8List attachmentLink;
  int preceivedTimestamp;
  int likes;
  int shares;
  int views;
  ModeratorCapsuleItem(
      this.flags,
      this.authorHint,
      this.bl,
      this.pp,
      this.garyTimestamp, //word, bigger is newer
      this.payloadLenght, //word
      this.payload, //under 1kb
      this.keyLen,
      this.key,
      this.hint,
      this.authorAvatar,
      this.authorNick,
      this.attachmentLink,
      this.lenghtInBytes,
      this.preceivedTimestamp,
      this.likes,
      this.shares,
      this.views);

  factory ModeratorCapsuleItem.fromBinary(Uint8List data) {
    var bdata = new ByteData.view(data.buffer);
    int bleng = bdata.getUint16(0);
    //2,3 flags
    //int tid = bdata.getUint16(0);
    int ah = bdata.getUint64(4);
    int bl = bdata.getUint64(12);
    int pp = bdata.getUint64(20);
    int gt = bdata.getUint16(28);
    int ple = bdata.getUint16(30);
    int timestamp = bdata.getUint64(32);
    //print('caps en tim ' + timestamp.toString());

    var wi = 40;
    //ByteData pload = bdata.buffer.asByteData(wi, wi + ple);
    Uint8List pload = data.sublist(wi, wi + ple);
    wi += ple;
    int kle = bdata.getInt16(wi);
    wi += 2;

    Uint8List kz = Uint8List(9999);

    bool kzin = false;
    if (kle > 0) {
      //key might no
      //t exist???
      kzin = true;
      int tle = wi + kle;
      kz = data.sublist(wi, tle);
      //print(tle);
      //print(data.lengthInBytes);
      //kz = bdata.buffer.asByteData(wi, tle);
      wi += kle;
    }
    int hi = bdata.getUint64(wi);
    wi += 8;
    //authornick,empty if 0
    String aav = '';
    var aal = bdata.getInt8(wi++);
    if (aal != 0) {
      //utf8.encode(navatar);
      aav = utf8.decode(data.sublist(wi, wi + aal).toList());
      wi += aal;
    }
    //author avatar, empty if 0
    String aan = '';
    var abl = bdata.getInt8(wi++);
    if (abl != 0) {
      try {
        aan = utf8.decode(data.sublist(wi, wi + abl).toList());
      } catch (e) {
        aan = '??';
      }
      wi += abl;
    }
    //attachmentLink
    Uint8List atl = Uint8List(16);
    int atl_l = 0;
    var atll = bdata.getInt8(wi++);
    if (atll != 0) {
      try {
        atl = data.sublist(wi, wi + atll);
        atl_l = atll;
      } catch (e) {}
      wi += atl_l;
    }
    int likes = 0;
    int shares = 0;
    int views = 0;

    likes = bdata.getUint16(wi);
    wi += 2;
    shares = bdata.getUint16(wi);
    wi += 2;
    if (wi < bdata.lengthInBytes) {
      views = bdata.getUint16(wi);
    }
    return ModeratorCapsuleItem(
        data.sublist(2, 4), //flags
        ah,
        bl,
        pp,
        gt,
        ple,
        pload.buffer.asUint8List(),
        kzin ? kz.buffer.lengthInBytes : 0,
        kzin ? kz.buffer.asUint8List() : Uint8List(0),
        hi,
        aav,
        aan,
        atl.sublist(0, atl_l),
        bleng,
        timestamp,
        likes,
        shares,
        views);
  }

  Uint8List get binary {
    Uint8List b = Uint8List(2048);
    var bdata = new ByteData.view(b.buffer);
    bdata.setUint8(2, flags[0]);
    bdata.setUint8(3, flags[1]);
    bdata.setUint64(4, authorHint);
    bdata.setUint64(12, bl);
    bdata.setUint64(20, pp);
    bdata.setUint16(28, garyTimestamp);
    bdata.setUint16(30, payload.lengthInBytes);
    bdata.setUint64(32, preceivedTimestamp);

    var wi = 40;
    payload.forEach((element) {
      bdata.setUint8(wi++, element);
    });
    //encrypted key for this thread
    //print(key);
    bdata.setUint16(wi, key.lengthInBytes);
    wi += 2;
    if (key.isNotEmpty) {
      key.forEach((element) {
        bdata.setUint8(wi++, element);
      });
    }
    //print('hint bin idx ' + wi.toString());
    bdata.setUint64(wi, hint);
    wi += 8;

    if (authorAvatar.isEmpty) {
      wi++;
    } else {
      List<int> aa = utf8.encode(authorAvatar);

      //var aa = authorAvatar.runes;
      bdata.setUint8(wi++, aa.length);
      aa.forEach((_r) {
        bdata.setUint8(wi++, _r);
      });
    }

    if (authorNick.isEmpty) {
      wi++;
    } else {
      List<int> aa = utf8.encode(authorNick);
      //var aa = authorNick.runes;
      bdata.setUint8(wi++, aa.length);
      aa.forEach((_r) {
        bdata.setUint8(wi++, _r);
      });
    }

    if (attachmentLink.isEmpty) {
      wi++;
    } else {
      /*List<int> aa = utf8.encode(attachmentLink);
      bdata.setUint8(wi++, aa.length);
      aa.forEach((_r) {
        bdata.setUint8(wi++, _r);
      });*/
      bdata.setUint8(wi++, attachmentLink.length);
      attachmentLink.forEach((_r) {
        bdata.setUint8(wi++, _r);
      });
    }
    //dont push 0
    /*print(wi);
    print('ptim ' + preceivedTimestamp.toString());
    bdata.setUint64(wi, preceivedTimestamp);
    var tot = bdata.getUint64(wi);
    if (preceivedTimestamp != tot) {
      print('PANI perc timestamp!!');
    }
    wi += 8;*/

    /*bdata.setUint16(0, wi);
    wi += 2;*/
    bdata.setUint16(wi, likes);
    wi += 2;
    bdata.setUint16(wi, shares);
    wi += 2;
    bdata.setUint16(wi, views);
    wi += 2;
    bdata.setUint16(0, wi);
    return b.sublist(0, wi);
  }
}
