import 'dart:convert';

import 'ModeratorEntryFlags.dart';
import 'dart:typed_data';
import '../interop/enums.dart';
import '../interop/xxhash/lib/fast_hash.dart';

class ModeratorEntry {
  int type;
  String author;
  Uint8List _body;
  Uint8List _shortlink;
  int keyHint;
  int bl;
  int pp;
  String colour;
  String avatar;
  String _nick;
  Uint8List attachmentLink;
  ModeratorEntryFlags flags;
  bool _bodyFucked = false;
  int perceivedTimestamp = 0;
  int likes = 0;
  int shares = 0;
  int views = 0;
  String siteUrl = '';
  /*bool isReply = false;
  bool isPost = false;
  bool isRoot = false;
  bool pawMatch = false;
  bool blsGroupMatch = false;
  bool muted;
  bool isGroupParent = false;
  bool isHidden = false;
  bool isHiddenDecrypted = true;
  bool isAnonymous = true;*/
  bool get bodyFucked => _bodyFucked;
  Map toJson() => {
        'key': base64Encode(_shortlink),
        'body': body,
        'nick': nick,
        'likes': likes,
        'shares': shares,
        'moderate': flags.moderate,
        'type': flags.isPost ? 'post' : 'reply',
        'ptime': perceivedTimestamp,
        'views': views
      };

  int get contentDupHash {
    List<int> g = [];

    g.addAll(attachmentLink);
    if (author.isNotEmpty) {
      g.addAll(utf8.encode(author));
    }

    g.addAll(_body);
    return xxHash(Uint8List.fromList(g));
  }

  List<String> get tags {
    List<String> _l = [];
    if (bodyFucked) return _l;
    var bd = body.trim().replaceAll("\n", " ");
    var bs = bd.split(" ");
    bs.forEach((element) {
      if (element.startsWith('#')) {
        var td = element.trim().split(' ');
        if (td.first.endsWith(',')) {
          _l.add(td[0].substring(1, td[0].length - 1));
        } else {
          _l.add(td[0].substring(1, td[0].length));
        }
      }
    });
    return _l;
  }

  String get softNickTagForSms {
    String s = softNick + ' ';
    if (tags.isNotEmpty) {
      s = s + '#' + tags.last;
    }
    return s;
  }

  String get cleanedBody {
    var tb = body.trim().split(' ');
    String n = '';

    tb.forEach((element) {
      if (element.startsWith('!')) {
        return;
      }
      n += element + ' ';
    });
    return n;
  }

  String get softNick {
    //nick might be bs because forgotten .about
    var tb = body.trim().split(' ');
    String n = nick;
    bool f = true;
    tb.forEach((element) {
      if (element.startsWith('/') && f) {
        n = element.substring(1, element.length);
        f = true;
      }
    });
    return n;
  }

  ModeratorEntry(
      this.type,
      this.author,
      this._body,
      this._shortlink,
      this.keyHint,
      this.bl,
      this.pp,
      this.colour,
      this.avatar,
      this._nick,
      this.attachmentLink,
      this.flags,
      this.perceivedTimestamp,
      this.likes,
      this.shares,
      this.views);
  Uint8List get bodyBytes => _body;
  Uint8List get shortLinkBytes => _shortlink;
  int _hintCache = 0;
  int get hint {
    if (_hintCache != 0) return _hintCache;
    //short link to crypted
    final digest = xxHash64.convert(_shortlink);
    _hintCache = digest.x.toInt();
    return _hintCache;
  }

  int get attachmentLinkAsXXint {
    if (!flags.attachements) return -1;
    //if (attachmentLink.isEmpty) return -1;
    var bdata = new ByteData.view(attachmentLink.buffer);
    var bdl = bdata.lengthInBytes;
    return bdata.getUint64(bdl - 8);
  }

  String get ipfsCid {
    var f = body.split(' ');
    var cid = '';
    f.forEach((element) {
      if (element.startsWith('!&')) {
        //might be followed immediately with another !| or..
        final bobo = element.split('!');
        cid = bobo[1].substring(1, bobo[1].length);
      }
    });
    return cid;
  }

  String get blurHash {
    var f = body.split(' ');
    var bh = '';
    f.forEach((element) {
      if (element.startsWith('!|')) {
        bh = element.substring(2, element.length);
      }
    });
    return bh;
  }

  String get nick {
    if (body.startsWith('/')) {
      var bda = body.split(' ').first;
      if (bda.length > 1) {
        return bda.substring(1, bda.length);
      }
    }
    if (_nick == null) {
      _nick = author.substring(0, 6);
    }
    return _nick;
  }

  int get typesetForCapsule {
    int i = 0;
    if (flags.isPost) i += 1; //0 reply
    if (flags.isGroup && !flags.isGroupMember) i += 4;
    return i;
  }

  int get timeElapsedSincePostMs {
    if (perceivedTimestamp == 0) return 0;
    return DateTime.now().millisecondsSinceEpoch - perceivedTimestamp;
  }

  bool get hasAttachment => flags.attachements;
  String _shortLinkCache = '';
  String get shortLink {
    if (_shortLinkCache.isNotEmpty) return _shortLinkCache;
    if (_shortlink.isEmpty) {
      _bodyFucked = true;
      return 'BROKEN_SLINK';
    }
    _shortLinkCache = base64UrlEncode(_shortlink);
    return _shortLinkCache;
    //return HtmlEscape().convert(utf8.decode(_shortlink));
  }

  String get body {
    if (_body.isEmpty) {
      _bodyFucked = true;
      return 'BROKEN_BODY';
    }
    try {
      var t = utf8.decode(_body);
      return t.trim();
    } catch (e) {
      _bodyFucked = true;
      return 'BROKEN_UTF8';
    }
  }

  void set nick(String s) => _nick = s;
  String get twitterPostTitle {
    //parse from body
    return '';
  }

  String get twitterPostContent {
    //grot of shitty tags
    return cleanedBody;
  }

  String get twitterAttachmentLink {
    if (ipfsCid.isEmpty) return '';
    return 'https://ipfs.io/ipfs/' + ipfsCid;
  }

  String get linkedImageUrlLink {
    final lil = body.split('![');
    if (lil.length < 2) return '';
    final lal = lil[1].split('(');
    final lol = lal[1].split(')');
    return lol[0];
  }

  int xxHash(Uint8List b) {
    if (b.isEmpty) return -1;
    final digest = xxHash64.convert(b);
    return digest.x.toInt();
  }

  String get debuObj {
    String s = '';
    if (this.flags.isPost) s += 'POST ';
    if (this.flags.isReply) s += 'REPLY ';
    s += 'type: ' + this.type.toString();
    s += ' auth: ' + this.author.toString();
    s += ' shortlink: ' + this._shortlink.lengthInBytes.toString();
    s += ' keyh: ' + this.keyHint.toString();
    s += ' bl: ' + this.bl.toString();
    s += ' pp: ' + this.pp.toString();
    s += ' avatar: ' + this.avatar.toString();
    s += ' nick: ' + this.nick.toString();
    s += ' atlink: ' + this.attachmentLink.toString();
    s += ' flags: ' + this.flags.toString();
    s += ' perTS: ' + this.perceivedTimestamp.toString();
    return s;
  }
}
