import 'dart:typed_data';
import 'ModeratorCapsuleEntry.dart';
import 'ModeratorEntryFlags.dart';
import 'package:palette/palette.dart';
import '../interop/enums.dart';
import 'dart:convert';
import 'ModeratorEntry.dart';
export 'ModeratorEntry.dart';

final ColorPalette _cp = ColorPalette.polyad(
  CmykColor(100, 50, 0, 0),
  numberOfColors: 127,
  hueVariability: 15,
  saturationVariability: 10,
  brightnessVariability: 3,
);

class ModeratorEntrySet {
  Map<int, ModeratorEntry> _data = {};
  Set<int> _groups = {};
  Set<int> _threads = {};
  Set<int> _posts = {};
  Set<int> _replies = {};
  Map<String, int> _tags = {};
  Map<String, List<int>> _taggedAs = {};
  Map<int, List<int>> _children = {};
  Map<int, List<int>> _groupChildren = {};
  Map<int, List<int>> _parentReplies = {};
  int idx = 0;
  ModeratorEntrySet();
  bool _sigilsChecked = false;
  Map<String, List<int>> _mentions = {};
  Map<String, List<int>> _blobs = {};
  Map<String, List<int>> _authors = {};
  List<int> _recent = [];
  bool _dirtyOrdering = false;
  Set<int> _contentDups = {};

  String get toJson {
    final db = _data.values.toList();
    return jsonEncode(db);
  }

  ModeratorEntry? read(String shortLink) {
    ModeratorEntry? ma;
    _data.forEach((key, value) {
      if (value.shortLink == shortLink) {
        ma = value;
      }
    });
    return ma;
  }

  bool markSeenByShortLink(String shortLink) {
    //ugly hack to patch cached
    bool hit = false;
    _data.forEach((key, value) {
      if (value.shortLink == shortLink) {
        if (!value.flags.isSeen) {
          value.flags.isSeen = true;
          hit = true;
        }
      }
    });
    return hit;
  }

  int findParentKey(ModeratorEntry m) {
    return m.bl;
  }

  int findRootByShortlink(String m) {
    int ma = -1;

    _data.forEach((key, value) {
      if (value.shortLink == m) {
        ma = key;
      }
    });
    return ma;
  }

  void _sigilIndex() {
    _data.forEach((key, value) {
      var x = value.body.split(' ');
      x.forEach((String element) {
        if (element.isEmpty) return;
        var eb = element.substring(1, element.length);
        if (element.startsWith('@')) {
          if (value.flags.isReply) {
            if (_mentions[eb] == null) _mentions[eb] = [];
            var mm = _data[value.bl];
            if (mm != null) {
              _mentions[eb]!.add(value.bl);
            }
          } else {
            if (_mentions[eb] == null) _mentions[eb] = [];
            _mentions[eb]!.add(key);
          }
        }
        if (element.startsWith('!&')) {
          if (_blobs[eb] == null) _blobs[eb] = [];
          _blobs[eb]!.add(key);
        }
        if (element.startsWith('/')) {
          if (_authors[eb] == null) _authors[eb] = [];
          _authors[eb]!.add(key);
        }
        if (element.startsWith('#')) {
          final g = element.substring(1, element.length);
          if (_tags[g] == null) _tags[g] = 0;
          _tags[g] = _tags[g]! + 1;
          if (_taggedAs[g] == null) {
            _taggedAs[g] = [];
          }
          _taggedAs[g]!.add(key);
          //_authors[eb]!.add(key);
        }
      });
    });
    _sigilsChecked = true;
  }

  List<ModeratorEntry> taggedAs(String a) {
    if (!_sigilsChecked) {
      _sigilIndex();
    }
    List<ModeratorEntry> _ll = [];
    if (_taggedAs[a] == null) return _ll;
    _taggedAs[a]!.forEach((element) {
      _ll.add(_data[element]!);
    });
    return _ll;
  }

  List<ModeratorEntry> mentioning(String a) {
    List<ModeratorEntry> _ll = [];
    if (_sigilsChecked) {
      if (_mentions[a] == null) return _ll;
      _mentions[a]!.forEach((element) {
        _ll.add(_data[element]!);
      });
      return _ll;
    }
    _sigilIndex();
    return mentioning(a);
  }

  List<ModeratorEntry> authoredBy(String a) {
    List<ModeratorEntry> _ll = [];
    if (_sigilsChecked) {
      if (_authors[a] == null) return _ll;
      _authors[a]!.forEach((element) {
        _ll.add(_data[element]!);
      });
      return _ll;
    }
    _sigilIndex();
    return authoredBy(a);
  }

  List<ModeratorEntry> authors(String a) {
    List<ModeratorEntry> _ll = [];
    if (_sigilsChecked) {
      if (_authors[a] == null) return _ll;
      var trix = _authors[a];
      _authors[a]!.forEach((element) {
        _ll.add(_data[element]!);
      });
      return _ll;
    }
    _sigilIndex();
    return authors(a);
  }

  List<ModeratorEntry> blobs(String a) {
    List<ModeratorEntry> _ll = [];
    if (_sigilsChecked) {
      _blobs[a]!.forEach((element) {
        _ll.add(_data[element]!);
      });
      return _ll;
    }
    _sigilIndex();
    return blobs(a);
  }

  bool _tagsChecked = false;
  Map<String, int> get tagsWithPopularity {
    if (_tagsChecked) return _tags;
    Map<String, int> _tll = {};
    _data.forEach((key, value) {
      var t = value.tags;
      if (t.isNotEmpty) {
        t.forEach((element) {
          if (_tll[element] == null) {
            _tll[element] = 1;
            _taggedAs[element] = [];
          } else {
            _tll[element] = _tll[element]! + 1;
            _taggedAs[element]!.add(key);
          }
        });
      }
    });
    _tagsChecked = true;
    _tags = _tll;
    return _tags;
  }

  factory ModeratorEntrySet.fromList(List<ModeratorEntry> _l) {
    ModeratorEntrySet _ll = ModeratorEntrySet();
    for (var element in _l) {
      _ll.add(element);
    }
    /*for (var element in _l) {
      _ll.add(element);
    }*/
    return _ll;
  }
  ModeratorEntry? get root {
    //case where we have only one root+replies
    if (_data.isEmpty) {
      print('mderaEntrySet get root; DATA empty, no root');
      return null;
    }
    ModeratorEntry? t;
    _data.forEach((key, value) {
      print(key);
      print(value.flags.isPost);

      if (value.flags.isPost) {
        t = value;
      }
    });
    return t;
  }

  List<ModeratorEntry> get all => _data.values.toList();
  List<ModeratorEntry> get posts =>
      all.where((element) => element.flags.isPost).toList();

  List<ModeratorEntry> get latestThreads {
    //find stuff with updated stuff possibly

    Map<int, List<int>> _sorta = {};
    List<int> tim = [];
    for (var i in _posts) {
      if (_data[i] == null) continue;
      ModeratorEntry? p = _data[i];
      if (_children[i] == null) {
        if (_sorta[p!.perceivedTimestamp] == null) {
          _sorta[p!.perceivedTimestamp] = [];
        }
        _sorta[p.perceivedTimestamp]!.add(i);

        continue;
      }
      var ct = p!.perceivedTimestamp;
      var cti = i;
      for (var ii in _children[i]!) {
        if (_data[ii] == null) continue;
        ModeratorEntry? cp = _data[ii];
        if (cp!.perceivedTimestamp > ct) {
          ct = cp.perceivedTimestamp;
          //cti = ii; keep ref to orig thread root
        }
      }
      //cti is the index to latest on that thread now
      if (_sorta[ct] == null) _sorta[ct] = [];
      _sorta[ct]!.add(cti);
    }
    //sort by keys
    var tl = _sorta.keys.toList();
    tl.sort();
    List<ModeratorEntry> _ll = [];
    for (var i in tl) {
      for (var ii in _sorta[i]!.reversed) {
        _ll.add(_data[ii]!);
      }
    }
    return _ll;
  }

  bool add(ModeratorEntry m) {
    if (m.bodyFucked) {
      return false;
    }
    if (m.shortLinkBytes.isEmpty) {
      return false;
    }

    final int hint = m.keyHint;
    bool updated = false;
    if (_data[m.keyHint] != null) {
      /*print(_data[m.keyHint]!.keyHint);
      print(m.bl);
      print(m.cleanedBody);
      if (_children[m.bl] != null) {
        print(_children[m.bl]);
      }*/
      if (m.hint != _data[m.keyHint]!.hint) {
        _data[m.keyHint] = m;
        updated = true;
      }
      /*if (m.perceivedTimestamp > 1 &&
          _data[m.keyHint]!.perceivedTimestamp < m.perceivedTimestamp) {
        _data[m.keyHint]!.perceivedTimestamp = m.perceivedTimestamp;
        updated = true;
      }*/
      if (_data[m.keyHint]!.likes != m.likes) {
        _data[m.keyHint]!.likes = m.likes;
        updated = true;
      }
      if (_data[m.keyHint]!.shares != m.shares) {
        _data[m.keyHint]!.shares = m.shares;
        updated = true;
      }
      if (_data[m.keyHint]!.views != m.views) {
        _data[m.keyHint]!.views = m.views;
        updated = true;
      }
      if (m.nick.isNotEmpty && _data[m.keyHint]!.nick != m.nick) {
        _data[m.keyHint]!.nick = m.nick;
        updated = true;
      }
      //_data[m.keyHint] = m;
      return updated;
    }

    final cdup = m.contentDupHash;
    //if (_contentDups.contains(cdup)) return false;
    _contentDups.add(cdup);

    _tagsChecked = false; //reindex tags
    _sigilsChecked = false;

    if (_children[m.keyHint] == null) {
      _children[m.keyHint] = [];
    }

    if (m.bl != 0) {
      if (_children[m.bl] == null) {
        _children[m.bl] = [];
      }
      _children[m.bl]!.add(m.keyHint);
    }
    if (m.flags.isGroup && !m.flags.isGroupMember) {
      _groups.add(m.keyHint);
      _children[m.keyHint] = [];

      _groupChildren[m.keyHint] = [];
    } else {
      if (m.flags.isPost) {
        _threads.add(m.keyHint);
      }
    }
    if (m.flags.isPost) {
      _posts.add(m.keyHint);
    }
    if (m.flags.isReply) {
      _replies.add(m.keyHint);
      if (m.bl != 0) {
        if (_children[m.bl] != null) {
          _children[m.bl]!.add(m.keyHint);
        }
        if (_parentReplies[m.bl] == null) {
          _parentReplies[m.bl] = [];
        }
        _parentReplies[m.bl]!.add(m.keyHint);
      }
    }

    _data[m.keyHint] = m;

    final mapEntries = _data.entries.toList()
      ..sort((a, b) =>
          a.value.perceivedTimestamp.compareTo(b.value.perceivedTimestamp));

    _data
      ..clear()
      ..addEntries(mapEntries.reversed);

    return true;
  }

  List<ModeratorEntry> get groups {
    List<ModeratorEntry> _g = [];
    _groups.forEach((element) {
      _g.add(_data[element]!);
    });
    return _g;
  }

  List<ModeratorEntry> get threads {
    List<ModeratorEntry> _g = [];
    _threads.forEach((element) {
      _g.add(_data[element]!);
    });
    return _g;
  }

  ModeratorEntry? currentThreadRoot;
  List<ModeratorEntry> get currentThread {
    List<ModeratorEntry> _ll = [];
    Set<int> _i = {};
    if (currentThreadRoot == null) return _ll;
    _ll.add(currentThreadRoot!);
    _i.add(currentThreadRoot!.hint);
    childrenOf(currentThreadRoot!.keyHint).forEach((element) {
      if (!element.flags.isReply) return;
      //if (_i.contains(element.hint)) return;
      _ll.add(element);
      _i.add(element.hint);
    });
    return _ll;
  }

  Set<ModeratorEntry> threadByShortLink(String s) {
    Set<ModeratorEntry> _ll = {};
    int r = findRootByShortlink(s);
    if (r == -1) return _ll;
    _ll.add(_data[r]!);
    childrenOf(r).forEach((element) {
      _ll.add(element);
    });

    //TODO sortapotty

    return _ll;
  }

  List<ModeratorEntry> childrenOf(int parent) {
    List<ModeratorEntry> _g = [];
    Set<int> _ix = {};
    if (_children[parent] == null) {
      return _g;
    }
    _children[parent]!.forEach((element) {
      if (!_ix.contains(element)) {
        _g.add(_data[element]!);
      }
      _ix.add(element);
    });
    return _g;
  }

  factory ModeratorEntrySet.fromModeratorCapsule(Uint8List data) {
    ModeratorEntrySet _ll = ModeratorEntrySet();
    List<ModeratorEntry> cai = [];

    //var wi = 0;
    int step = 0;
    var bdata = new ByteData.view(data.buffer);
    int wi = 0;
    while (wi < data.lengthInBytes) {
      int ll = bdata.getUint16(wi);
      var m =
          ModeratorCapsuleItem.fromBinary(data.sublist(wi, data.lengthInBytes));
      var mf = ModeratorEntryFlags(m.flags);

      cai.add(ModeratorEntry(
          ThicketContainerSubType.about.index,
          '',
          m.payload,
          m.key,
          m.hint,
          m.bl,
          m.pp,
          '',
          m.authorAvatar,
          m.authorNick,
          m.attachmentLink,
          mf,
          m.preceivedTimestamp,
          m.likes,
          m.shares,
          m.views));
      wi += ll;
    }
    /*cai.forEach((element) {
      print(element.debuObj);
    });*/
    return ModeratorEntrySet.fromList(cai);
  }
  //get the binary with helper
}

/*
class ThicketEntryHelper {
  Tentry? parse(ThicketEntry t, Set<int> _group) {
    if (!t.isSns) return null;
    
    tp.author = t.author.base58;
    tp.body = t.payloadAsString;
    // markdownToHtml(t.payloadAsString);
    tp.isReply = t.isReply;
    tp.link = t.entryHash.base58;
    if (tp.backLink != null) {
      if (_group.contains(t.backLink.hint)) {
        tp.backLink = t.backLink.base58;
      }
    }
    return Tentry(type, author, body, link, backLink, colour, avatar, _nick)
  }
}*/
